/* JOTTO is a simple but entertaining word game.  I based this LASER C
	version on a simpler version for the Commodore 64 (in BASIC) by Jim
	Butterfield.
		This program was an experiment for me to see if I could write a
	game that worked in different resolutions.  I've always been bugged
	by games that complain about the resolution.  If you're not using GEM 
	drop-down menus and whatnot, there shouldn't be any reason to insist 
	on the player rebooting.  I didn't need to make this a .PRG (it really
	is just a .TOS program), but I also wanted to try turning the mouse
	off and back on--another thing that bugs me about some programs is how
	they mishandle the mouse. (Lecture over.)
		Anyway, all grumbling aside, you might find the source interesting
	for how to handle resolution changes, play with the color palette, use
	VT-52 codes, and you might even enjoy the game!  I make no claim for
	particularly good programming practices (I'm a Fortran programmer at
	heart), and would appreciate any comments, criticisms, suggestions for
	better coding, etc.  I can most easily be reached via GEnie--address
	to SESCHNEIDER--or on Fidonet as Steve Schneider.  
	     This game and source code are hereby released for free distribution
	with the stipulation that no other copyright may be placed upon the 
	source code or program, and these notes must remain with the source code.
	Obviously, since this program is free, I warrant it to the extent that
	you paid for it, i.e. not at all.
*/

/* Known problems:  Program exits with status 4 according to NeoDesk
   but this doesn't seem to affect anything.  Can anyone tell me why?
   I'd also like to know why the setscreen() function doesn't work
   as my documentation indicates it should--I can't simply set the
   physical and logical screen bases to negative values when the
   resolution is switched. */

#include <stdio.h>         /* Standard IO declarations */
#include <osbind.h>        /* TOS bindings */
#include <strings.h>

#define NWORDS	50	/* number of puzzle words */
#define ESC		27
#define NUL		0

#define	TRUE	1
#define	FALSE	0

/* color palette we'll be using */
#define BLACK   0	/* BLACK in Monochrome (using colorfactor) */
#define BLUE	1	/* BLACK in Monochrome */
#define GOLD	2	/* WHITE in Monochrome */
#define GREEN	3	/* WHITE in Monochrome */

/* VT-52 codes used in this program */
	char	inverse[3]		= {ESC,'p',NUL},	/* reverse character an background colors */
			normal[3]		= {ESC,'q',NUL},	/* return to normal colors */
			charcolor[4]	= {ESC,'b',16,NUL},	/* used by setchar() to set color of characters */
			bkgdcolor[4]	= {ESC,'c',16,NUL}, /* used by setbkgd() to set color of background */
			clearaft1[7]	= {ESC,'H',ESC,'B',ESC,'J',NUL},  /* used by clrbot() to clear all but top line */
			clearscrn[3]	= {ESC,'E',NUL},		/* used by clrscrn() */
			locate[5]		= {ESC,'Y',32,32,NUL},	/* used by prtxy() to position text */
			crsr0[3]		= {ESC,'f',NUL},	/* used by crsroff() to turn off cursor */
			crsr1[3]		= {ESC,'e',NUL},	/* used by crsron() to turn on cursor */
			nowrap[3]		= {ESC,'w',NUL};	/* don't wrap at end of lines--some progs turn this on and
													forget to turn it off again */

/* Screen and resolution parameters */
char	*scrnbase,	/* physical screen base */
		*scrnlog;	/* logical screen base */
int		oldpalette[16],
		oldrez,
		colorfactor;

/* encoded puzzle words (doesn't take a genius to figure out the code!)*/
char	*ans[] = {	"BXBSF", "IPCCZ", "DBDIF", "ESFBE", "PGGBM",
					"HPSHF", "IBUDI", "DJWJM", "KPMMZ", "LBZBL",
					"SBKBI", "MFWFM", "NJNJD", "BOOFY", "QJQFS",
					"RVFTU", "SJWFS", "QSFTT", "PUUFS", "FWFOU",
					"XFBWF", "FYUPM", "NVTIZ", "AFCSB", "GJAAZ",
					"UIJDL", "ESVOL", "GMPPE", "UJHFS", "GBLFS",
					"CPPUI", "MZJOH", "TRVBU", "HBVAF", "PXJOH",
					"UISFF", "TJHIU", "BYMFT", "HSVNQ", "BSFOB",
					"RVBSU", "DSFFQ", "CFMDI", "QSFTT", "TQBSL",
					"SBEBS", "SVSBM", "TNFMM", "GSPXO", "ESJGU"
				};

/* global variables and functions with return values */
int		i,	/* utility variable */
		row,
		column,
		checkguess();
char	string[80];	/* utility variable	*/

 /*****************************************************************/
/*****************************main program************************/
main()
{
	char	yesno,
			guess[10],
			answer[6];
	int		guessnumber,
			guessagain,
			inputerror;
	long	seed;

/* save info on current screen, and set to medium rez */
	setscreen();

/* buffered output to the screen is a pain--turn it off */
	setbuf(stdout,NULL);

/* print startup screen */
	instructions();

/* initialize random number parameters */
	seed = Random();
	srand(seed);

/* main game loop */
	do
	{
	/* choose a word at random from list */
		chooseword(answer);
		guessnumber = 0;
	/* guess loop (keep guessing 30 times or until correct) */
		do
		{
		/* get new guesses while 'guessagain' is TRUE */
			guessnumber += 1;
			/* set row and column for screen output */
			setrowcol(guessnumber);
		/* input loop (test for valid input) */
			do
			{
			/* keep asking for a new guess until it's acceptable */
				sprintf(string,"   Enter guess %d: ",guessnumber);
				prtxy(string,column,row);
				getguess(guess);

				/* allow the player to exit gracefully with "quit" */
				if ( strncmp(guess,"QUIT",5)==0 )
					exitgame();

				/* check if the answer is correct, otherwise examine guess */
				if ( strncmp(guess,answer,6)==0 )
				{
					gotitright(guessnumber); /* we have a winner! */
					guessagain = FALSE;
					row += 3;
					inputerror = FALSE;
				}
				else if ( !(inputerror=checkguess(guess)) )
				{
					countjots(guess,answer,guessnumber);

					if (guessnumber == 30) /* cut'm off--he's hopeless */
					{
						condolences(answer);
						guessagain = FALSE;
						row += 4;
					}
					else
						guessagain = TRUE;
				}
			} while (inputerror);
		} while (guessagain);
	} while (playagain());

	exitgame();
}

 /*****************************************************************/
/*******setscreen -- save screen parameters and switch rez********/
setscreen()
{
	scrnbase = (char*)Physbase();
	scrnlog  = (char*)Logbase();
	for(i=0;i<16;i++)
		oldpalette[i] = Setcolor(i,-1);
	if ((oldrez = Getrez()) == 0)
		Setscreen(scrnlog,scrnbase,1);
	if (oldrez == 2)
		colorfactor = 2;
	else
		colorfactor = 1;
/* choose new palette colors */
	Setcolor(0,0x000);	/* black	*/
	Setcolor(1,0x014);	/* blue		*/
	Setcolor(2,0x752);	/* gold 	*/
	Setcolor(3,0x074);	/* green	*/
	a_hidemouse();	/* line-a function to remove mouse from screen */
}

 /*****************************************************************/
/*******instructions -- print start-up screen and offer help******/
instructions()
{
	char yesno;

	printf("%s",nowrap);
	crsroff();
	clrscrn();
	setbkgd(BLACK);
	setchar(GOLD);
	printf("%s                                   J O T T O !                       ",normal);
	setchar(BLUE);
	printf("v0.9 by ses\n");

	setbkgd(GREEN);
	setchar(BLUE);
	clrbot();
	printf("Do you want instructions? (y/n) ");
	crsron();
	if ( (yesno=Cconin()) == 'y' || yesno == 'Y' )
	{
		clrbot();
		crsroff();
		printf(" Try to guess the mystery 5-letter word.  You may only guess legal 5-letter    \n");
		printf(" words.  You will be told the number of matches or \"jots\" for your guess (i.e.\n");
		printf(" the total number of letters your guess has in common with the answer.)\n");
		printf("    HINT: The trick is to vary slightly from one guess to the next--if you    \n");
		printf("    guess BATCH and get 2 jots, you might try BOTCH or CHART the next guess.\n");
		setchar(GOLD);
		printf("                         <Press any key to begin>");
		setchar(BLUE);
		Crawcin();
	}
}

 /*****************************************************************/
/********chooseword -- select a five letter word at random********/
chooseword(answer)
	char	*answer;
{
	int choice;

	choice = rand();
	if (choice < 0)
		choice *= -1;
	while (choice > NWORDS-1)
		choice -= NWORDS;
	strcpy (answer,ans[choice]);
	for (i=0;i<5;i++)
	{
		answer[i] -= 1;
		if (answer[i] == '@')
			answer[i] = 'Z';
	}
	clrbot();
	crsron();
	printf("%sI have a five letter word...  Guess (with legal words) and I'll tell you how    \n",inverse);
	printf(  "many jots you have. (Enter QUIT to quit.)                                       %s\n",normal);
}

 /*****************************************************************/
/******getguess -- get player input without messing up screen*****/
getguess(guess)
	char	*guess;
{
	char	ch;
	int		i;
	i = 0;
	while ( ((ch=Cnecin) != 13) && (i<9 || ch==8 || ch==127) )
	{
		if (ch > 32 && ch < 128)
		{
			Cconout(ch);
			guess[i++] = toupper(ch); /* use uppercase */
		}
 		/* backspace over mistakes with BS or DEL */
 		else if (ch==8 || ch==127)
		{
			if(--i > -1) /* don't backspace past beginning */
				Cconout(8);
			else
				i = 0;
		}
	}
	if (i>0)
	{
		if (i > 9)
			i=9;
		guess[i] = '\0';
	}
	else
	{
		guess[0]=' ';
		guess[1]='\0';
	}
	return;
}

 /*****************************************************************/
/**********gotitright -- fireworks for a correct answer***********/
gotitright(guess)
	int		guess;
{
	crsroff();
	sprintf (string,"  !!!!!!!!!!!!!  JOTTO  !!!!!!!!!!!!  ");
	for(i=0;i<9;i++)
	{
		setchar(GOLD);
		Setcolor(2,0x777);    /* white  */
		prtxy(string,column,row+1);
		setchar(BLACK);
		Setcolor(2,0x752);	  /* gold	*/
		prtxy(string,column,row+1);
	} /* ...this program really needs some sound effects... */
	setchar(BLUE);
	sprintf (string,"   ...in only %d turns!",guess);
	prtxy(string,column,row+2);
}

 /*****************************************************************/
/********* checkguess -- make sure the input is sensible *********/
int checkguess(guess)
	char	*guess;
{
	char	guess0[10],
			letter;
	int		vowels;

	vowels = 0;

	strncpy(guess0,guess,10);
	if (strlen(guess) != 5)
	{
		sprintf(string,"   %.10s is NOT a 5-letter guess!    ",guess0);
		prtxy(string,column,row+1);
		return(1);
	}
	for ( i=0 ; i<5 ; i++ )
	{
		letter = guess[i];
		/* check that there are only letters in the guess*/
		if ( letter<'A' || letter>'Z' )
		{
			sprintf(string,"   %s is NOT a word!!!!          ",guess0);
			prtxy(string,column,row+1);
			return(1);
		}
		if (letter=='A' || letter=='E' || letter=='I' || letter=='O' || letter=='U')
			vowels+=1;
	}
	/* there'd better be at least one vowel and one consonant */
	if (vowels==0 || vowels==5)
	{
		sprintf(string,"   What kind of word is %s?!    ",guess0);
		prtxy(string,column,row+1);
		return(1);
	}
	return(0);
}

 /*****************************************************************/
/*************countjots -- see how many letters match*************/
countjots(guess,answer,guessnumber)
	char	*guess,
			*answer;
	int		guessnumber;
{
	char	testguess[6],
			testans[6];
	int		jots,
			j;

/* clear out any previous error messages */
	prtxy("                                      ",column,row+1);

	strncpy (testans,answer,5);
	strncpy (testguess,guess,5);
	jots=0;

	for ( i=0 ; i<5 ; i++ )
	{
		for ( j=0 ; j<5 ; j++ )
		{
			if ( testguess[i]==testans[j] )
			{
				jots += 1;
				testans[j] = 1;
				testguess[i] = 2;
			}
		}
	}
	sprintf (string,"Guess %2d: ",guessnumber);
	prtxy (string,column,row);
	setchar(BLACK);
	printf ("%5s -- %d",guess,jots);
	setchar(BLUE);
	if (jots == 1)
		printf (" jot           ");
	else
		printf (" jots          ");
	}

 /*****************************************************************/
/*************condolences -- give up after 30 guesses*************/
condolences(answer)
	char	*answer;
{
	crsroff();
	prtxy ("...oh well, I'd better tell you the",column,row+1);
	sprintf (string,"mystery word.  It was %s %5s %s",inverse,answer,normal);
	prtxy (string,column,row+2);
	prtxy ("Better luck next time!     ",column,row+3);
}

 /*****************************************************************/
/****setrowcol -- choose output position according to guess # ****/
setrowcol(guessnumber)
	int		guessnumber;
{
	if (guessnumber < 18)
	{
		column = 1;
		row = guessnumber+4;
	}
	else
	{
		column = 2;
		row = guessnumber-13;
	}
}

 /*****************************************************************/
/*******playagain -- see if the player wants to play again********/
int playagain()
{
	char	yesno;
	int		play;

	prtxy ("Play again? (y/n) ",column,row);
	crsron();
	/* accept only 'y' or 'n' to avoid accidental exit */
	do
	{
		if ( (yesno=Crawcin()) == 'n' || yesno == 'N' )
			play = 0;
		else if (yesno=='y' || yesno == 'Y' )
			play = 1;
		else
			play = -1;
	} while (play < 0);
	clrbot();
	return(play);
}

 /*****************************************************************/
/*******exitgame -- restore old screen parameters and exit********/
exitgame()
{
	crsroff(); /* ever see one of those annoying cursors blinking in
					the middle of nowhere? Leaving this out will do it */
	setbkgd(0); /* restore defaults, or you'll get funny output */
	setchar(3);	/*     when listing files later*/
	Setpalette(oldpalette); /* restore previous colors */
	if (oldrez==0)
		Setscreen(scrnlog,scrnbase,0);
	a_showmouse();	/* you must balance your hidemouse()'s */
	exit();
}

 /*****************************************************************/
/*******setbkgd setchar crsron crsroff clrscrn clrbot prtxy*******/
/*                     VT-52 utility functions                  */
/*****setbkgd -- set background color */
setbkgd(color)
	int	color;
{
	bkgdcolor[2] = color/colorfactor+16;
	Cconws(bkgdcolor);
}
/*****setchar -- set character color */
setchar(color)
	int	color;
{
	charcolor[2] = color/colorfactor+16;
	Cconws(charcolor);
}
/*****crsroff -- turn off blinking cursor */
crsroff()
{
	Cconws(crsr0);
/* turning the cursor off makes for cleaner graphics and should be
   done before returning to the desktop */
}
/*****crsron -- turn on blinking cursor (prompt for input)*/
crsron()
{
	Cconws(crsr1);
}
/*****clrscrn -- clear entire screen, put cursor in home position */
clrscrn()
{
	Cconws(clearscrn);
}
/*****clrbot -- clear screen below title line */
clrbot()
{
	Cconws(clearaft1);
}
/*****prtxy -- position output at different positions on screen */
prtxy(string,x,y)
	char	*string;
	int		x,
			y;
{
	locate[2] = 32 + (y-1);
	locate[3] = 32 + (x-1)*40;
	printf("%s",locate);
	printf("%s",string);
	return;
}
